@@ -23,14 +23,14 @@ DOCUMENTATION = '''
 ---
 module: unarchive
 version_added: 1.4
-short_description: Copies an archive to a remote location and unpack it
+short_description: Unpacks an archive after (optionally) copying it from the local machine.
 extends_documentation_fragment: files
 description:
-     - The M(unarchive) module copies an archive file from the local machine to a remote and unpacks it.
+     - The M(unarchive) module unpacks an archive. By default, it will copy the source file from the local system to the target before unpacking - set copy=no to unpack an archive which already exists on the target..
 options:
   src:
     description:
-      - Local path to archive file to copy to the remote server; can be absolute or relative.
+      - If copy=yes (default), local path to archive file to copy to the target server; can be absolute or relative. If copy=no, path on the target server to existing archive file to unpack.
     required: true
     default: null
   dest:
@@ -40,7 +40,7 @@ options:
     default: null
   copy:
     description:
-      - "if true, the file is copied from the 'master' to the target machine, otherwise, the plugin will look for src archive at the target machine."
+      - "If true, the file is copied from local 'master' to the target machine, otherwise, the plugin will look for src archive at the target machine."
     required: false
     choices: [ "yes", "no" ]
     default: "yes"
@@ -76,18 +76,35 @@ EXAMPLES = '''
 '''
 
 import os
+from zipfile import ZipFile
 
+class UnarchiveError(Exception):
+    pass
 
 # class to handle .zip files
-class ZipFile(object):
-    
+class ZipArchive(object):
+
     def __init__(self, src, dest, module):
         self.src = src
         self.dest = dest
         self.module = module
         self.cmd_path = self.module.get_bin_path('unzip')
+        self._files_in_archive = []
 
-    def is_unarchived(self):
+    @property
+    def files_in_archive(self, force_refresh=False):
+        if self._files_in_archive and not force_refresh:
+            return self._files_in_archive
+
+        archive = ZipFile(self.src)
+        try:
+            self._files_in_archive = archive.namelist()
+        except:
+            raise UnarchiveError('Unable to list files in the archive')
+
+        return self._files_in_archive
+
+    def is_unarchived(self, mode, owner, group):
         return dict(unarchived=False)
 
     def unarchive(self):
@@ -106,8 +123,8 @@ class ZipFile(object):
 
 
 # class to handle gzipped tar files
-class TgzFile(object):
-    
+class TgzArchive(object):
+
     def __init__(self, src, dest, module):
         self.src = src
         self.dest = dest
@@ -118,11 +135,49 @@ class TgzFile(object):
             # Fallback to tar
             self.cmd_path = self.module.get_bin_path('tar')
         self.zipflag = 'z'
+        self._files_in_archive = []
 
-    def is_unarchived(self):
-        cmd = '%s -v -C "%s" --diff -%sf "%s"' % (self.cmd_path, self.dest, self.zipflag, self.src)
+    @property
+    def files_in_archive(self, force_refresh=False):
+        if self._files_in_archive and not force_refresh:
+            return self._files_in_archive
+
+        cmd = '%s -t%sf "%s"' % (self.cmd_path, self.zipflag, self.src)
+        rc, out, err = self.module.run_command(cmd)
+        if rc != 0:
+            raise UnarchiveError('Unable to list files in the archive')
+
+        for filename in out.splitlines():
+            if filename:
+                self._files_in_archive.append(filename)
+        return self._files_in_archive
+
+    def is_unarchived(self, mode, owner, group):
+        cmd = '%s -C "%s" --diff -%sf "%s"' % (self.cmd_path, self.dest, self.zipflag, self.src)
         rc, out, err = self.module.run_command(cmd)
         unarchived = (rc == 0)
+        if not unarchived:
+            # Check whether the differences are in something that we're
+            # setting anyway
+
+            # What will be set
+            to_be_set = set()
+            for perm in (('Mode', mode), ('Gid', group), ('Uid', owner)):
+                if perm[1] is not None:
+                    to_be_set.add(perm[0])
+
+            # What is different
+            changes = set()
+            difference_re = re.compile(r': (.*) differs$')
+            for line in out.splitlines():
+                match = difference_re.search(line)
+                if not match:
+                    # Unknown tar output. Assume we have changes
+                    return dict(unarchived=unarchived, rc=rc, out=out, err=err, cmd=cmd)
+                changes.add(match.groups()[0])
+
+            if changes and changes.issubset(to_be_set):
+                unarchived = True
         return dict(unarchived=unarchived, rc=rc, out=out, err=err, cmd=cmd)
 
     def unarchive(self):
@@ -133,47 +188,41 @@ class TgzFile(object):
     def can_handle_archive(self):
         if not self.cmd_path:
             return False
-        cmd = '%s -t%sf "%s"' % (self.cmd_path, self.zipflag, self.src)
-        rc, out, err = self.module.run_command(cmd)
-        if rc == 0:
-            if len(out.splitlines(True)) > 0:
+
+        try:
+            if self.files_in_archive:
                 return True
+        except UnarchiveError:
+            pass
+        # Errors and no files in archive assume that we weren't able to
+        # properly unarchive it
         return False
 
 
 # class to handle tar files that aren't compressed
-class TarFile(TgzFile):
+class TarArchive(TgzArchive):
     def __init__(self, src, dest, module):
-        self.src = src
-        self.dest = dest
-        self.module = module
-        self.cmd_path = self.module.get_bin_path('tar')
+        super(TarArchive, self).__init__(src, dest, module)
         self.zipflag = ''
 
 
 # class to handle bzip2 compressed tar files
-class TarBzip(TgzFile):
+class TarBzipArchive(TgzArchive):
     def __init__(self, src, dest, module):
-        self.src = src
-        self.dest = dest
-        self.module = module
-        self.cmd_path = self.module.get_bin_path('tar')
+        super(TarBzipArchive, self).__init__(src, dest, module)
         self.zipflag = 'j'
 
 
 # class to handle xz compressed tar files
-class TarXz(TgzFile):
+class TarXzArchive(TgzArchive):
     def __init__(self, src, dest, module):
-        self.src = src
-        self.dest = dest
-        self.module = module
-        self.cmd_path = self.module.get_bin_path('tar')
+        super(TarXzArchive, self).__init__(src, dest, module)
         self.zipflag = 'J'
 
 
 # try handlers in order and return the one that works or bail if none work
 def pick_handler(src, dest, module):
-    handlers = [TgzFile, ZipFile, TarFile, TarBzip, TarXz]
+    handlers = [TgzArchive, ZipArchive, TarArchive, TarBzipArchive, TarXzArchive]
     for handler in handlers:
         obj = handler(src, dest, module)
         if obj.can_handle_archive():
@@ -197,6 +246,7 @@ def main():
     src    = os.path.expanduser(module.params['src'])
     dest   = os.path.expanduser(module.params['dest'])
     copy   = module.params['copy']
+    file_args = module.load_file_common_arguments(module.params)
 
     # did tar file arrive?
     if not os.path.exists(src):
@@ -218,23 +268,29 @@ def main():
     res_args = dict(handler=handler.__class__.__name__, dest=dest, src=src)
 
     # do we need to do unpack?
-    res_args['check_results'] = handler.is_unarchived()
+    res_args['check_results'] = handler.is_unarchived(file_args['mode'],
+            file_args['owner'], file_args['group'])
     if res_args['check_results']['unarchived']:
         res_args['changed'] = False
-        module.exit_json(**res_args)
+    else:
+        # do the unpack
+        try:
+            res_args['extract_results'] = handler.unarchive()
+            if res_args['extract_results']['rc'] != 0:
+                module.fail_json(msg="failed to unpack %s to %s" % (src, dest), **res_args)
+        except IOError:
+            module.fail_json(msg="failed to unpack %s to %s" % (src, dest))
+        else:
+            res_args['changed'] = True
 
-    # do the unpack
-    try:
-        res_args['extract_results'] = handler.unarchive()
-        if res_args['extract_results']['rc'] != 0:
-            module.fail_json(msg="failed to unpack %s to %s" % (src, dest), **res_args)
-    except IOError:
-        module.fail_json(msg="failed to unpack %s to %s" % (src, dest))
-
-    res_args['changed'] = True
+    # do we need to change perms?
+    for filename in handler.files_in_archive:
+        file_args['path'] = os.path.join(dest, filename)
+        res_args['changed'] = module.set_fs_attributes_if_different(file_args, res_args['changed'])
 
     module.exit_json(**res_args)
 
 # import module snippets
 from ansible.module_utils.basic import *
-main()
+if __name__ == '__main__':
+    main()
