@@ -304,7 +304,7 @@ def uri(module, url, dest, user, password, body, method, headers, redirects, soc
         r.update(resp_redir)
         r.update(resp)
         try:
-            return r, unicode(content.decode('unicode_escape')), dest
+            return r, unicode(content.decode('raw_unicode_escape')), dest
         except:
             return r, content, dest
     except httplib2.RedirectMissingLocation:
@@ -381,7 +381,7 @@ def main():
         # of uri executions.
         creates = os.path.expanduser(creates)
         if os.path.exists(creates):
-            module.exit_json(stdout="skipped, since %s exists" % creates, skipped=True, changed=False, stderr=False, rc=0)
+            module.exit_json(stdout="skipped, since %s exists" % creates, changed=False, stderr=False, rc=0)
 
     if removes is not None:
         # do not run the command if the line contains removes=filename
@@ -389,7 +389,7 @@ def main():
         # of uri executions.
         v = os.path.expanduser(removes)
         if not os.path.exists(removes):
-            module.exit_json(stdout="skipped, since %s does not exist" % removes, skipped=True, changed=False, stderr=False, rc=0)
+            module.exit_json(stdout="skipped, since %s does not exist" % removes, changed=False, stderr=False, rc=0)
 
 
     # httplib2 only sends authentication after the server asks for it with a 401.
@@ -426,7 +426,8 @@ def main():
         uresp[ukey] = value
 
     if 'content_type' in uresp:
-        if uresp['content_type'].startswith('application/json'):
+        if uresp['content_type'].startswith('application/json') or \
+                uresp['content_type'].startswith('text/json'):
             try:
                 js = json.loads(content)
                 uresp['json'] = js
